

strong {
    font-weight: bold;
    font-size: var(--font-size-medium);
}

input, textarea, select, file {
    background-color: var(--form-in-bg-color);
    border-radius: var(--border-radius);
    color: var(--form-color); /* Text color */
    border: 3px solid var(--form-border-color); /* Border color */
    margin: 2px;
    padding: 0px; /* Padding inside the fields */
    /* ... other styles ... */
}

body {
    font-family: var(--font-family);
    background-color: var(--background-color);
    color: var(--text-color);
    font-size: var(--font-size-standard);
    font-weight: inherit; 
}

.container {
    margin-top: 20px;
    background-color: var(--container-bg-color);
    padding: 20px;
    border-radius: var(--border-radius);
    width: 100%;
}

.field {
    font-size: var(--font-size-large);
    font-weight: bold;
    padding: 20px;
    color: var(--text-color);
}

#ratio {
    font-size: var(--font-size-xlarge);
    font-weight: bold;
    margin: 20px 0;
    color: var(--text-color);
}

.info-box {
    display: none;
    position: flex;
    top: 10px;
    right: 10px;
    border: 2px solid var(--info-box-border-color);
    background: inherit;
    padding: 10px;
    border-radius: var(--border-radius);
    z-index: 100;
    font-size: var(--font-size-standard);
}

/* General Button Style */
button {
    font-size: var(--font-size-standard);
    font-weight: inherit; 
    font-family: var(--font-family);
    background-color: var(--primary-color);
    color: var(--text-color);
    border: none;
    border-radius: var(--border-radius);
    cursor: pointer;
    margin: 2px;
    transition: background-color var(--transition-speed);
}

button:hover {
    background-color: var(--primary-hover-color);
}

.idx_button:hover {
    background-color: var(--primary-hover-color);
}

.idx_button {
    font-size: var(--font-size-standard);
    background-color: var(--primary-color);
    color: var(--text-color);
    border: none;
    border-radius: var(--border-radius);
    cursor: pointer;
    margin: 1px;
    transition: background-color var(--transition-speed);
}
    
form {
    padding: 0px;
    margin: 2px;
}

table {
    border-collapse: collapse;
    width: 100%;
    color: var(--text-color);
    font-size: var(--font-size-standard);
    text-align: left;
}

.accordion {
    background-color: var(--secondary-color);
    color: var(--text-color);
    cursor: pointer;
    text-align: left;
    border: none;
    outline: none;
    transition: var(--transition-speed);
}

.active, .accordion:hover {
    background-color: var(--accent-color);
}

.accordion:after {
    content: '\002B';
    color: var(--accordion-after-color);
    font-weight: bold;
    float: right;
    margin-left: 5px;
}

.active:after {
    content: "\2212";
}

.form-row {
    display: flex;
    align-items: center;
}

.form-label {
    text-align: right;
}

.form-input {
    
}

.list-input-container {
    display: flex;
    flex-wrap: wrap;
    gap: 5px;
}

.list-input-item {
    min-width: 150px;
}

.list-action-buttons {
    display: flex;
    align-items: center;
    gap: 5px;
    margin-top: 5px;
}

.form-container {
    display: grid;
    grid-template-columns: repeat(2, 1fr);
    gap: 10px;
    margin-bottom: 20px;
}

.form-container label {
    display: block;
}

.form-container input[type="text"] {
    width: 100%;
    border: 1px solid var(--form-container-input-color);
    border-radius: var(--border-radius);
}

.form-button {
    background-color: var(--addl-button-color);
    color: var(--text-color);
    border: none;
    border-radius: var(--border-radius);
    cursor: pointer;
}

.form-button:hover {
    background-color: var(--primary-hover-color);
}

.content {
    padding: 0 18px;
    display: none;
    overflow: hidden;
}

.status_button, .action_button, #property_button, .wf_action_button {
    font-size: var(--font-size-xsmall);
    padding: 3px 6px;
    background-color: var(--secondary-color);
    color: var(--status-button-color);
    border: none;
    border-radius: var(--border-radius);
    cursor: pointer;
    transition: background-color var(--transition-speed);
    margin-top: 5px;
}

.action_button, .wf_action_button {
    background-color: var(--primary-color);
    color: var(--text-color);
}

#property_button {
    background-color: var(--property-button-color); 
    color: var(--text-color);
}

.wf_action_button {
    background-color: var(--primary-color);
    color: var(--text-color);
    font-size: var(--font-size-standard);
    padding: 3px 6px;
}


.json-key {
    font-weight: bold;
}
.json-addl {
    margin-left: 20px;
}

p {
    margin: 0;
    display: inherit;

}
.highlight {
    background-color: var(--highlight-color);
}
.floating-button {
    position: fixed;
    bottom: 20px;
    right: 20px;
    margin: 5px;
    padding: 10px;
    font-size: 24px; /* Adjust the size as needed */
    text-align: center;
    line-height: 30px; /* Adjust for vertical alignment */
    width: 50px; /* Fixed width */
    height: 50px; /* Fixed height */
    border-radius: 50%; /* Optional: makes the button circular */
    background-color: var(--primary-color);
    border: none; /* Optional: removes border */
    cursor: pointer;
    z-index: 1000;
}
 
.floating-button2 {   
    position: fixed;
    right: 20px;
    margin: 10px;
    padding: 10px;
    border: none;
    border-radius: 5px;
    background-color:  var(--primary-color);
    color: white;
    cursor: pointer;
    transition: background-color 0.3s;
}

.floating-button2:hover {
    background-color: var(--secondary-color);

}

.floating-button-logout2 {
    bottom: 80px;  /* Adjusted for spacing */
}

.floating-button-skin2 {
    bottom: 20px;
}

#collapseAll {
    right: 80px; /* Adjust this value to position the collapse button */
}

#euidInput, #distanceSlider {
    margin: 10px;
}
/* Normal link color */
a {
    color: var(--a); /* Change to your preferred color */
    text-decoration: none; /* Removes underline from links */
}

/* Visited link color */
a:visited {
    color: var(--a-visited); /* Change to your preferred color */
}

/* Hover link color */
a:hover {
    color: var(--a-hover); /* Change to your preferred color */
    text-decoration: underline; /* Adds underline on hover */
}

/* Active link color (when clicked) */
a:active {
    color: var(--a-active); /* Change to your preferred color */
}

.subtype-button {
    font-size: var(--font-size-xsmall);
    padding: 1px 2px;
    background-color:  rgb(22, 65, 145);
    color: #d6f3f8;
    border: none;
    border-radius: var(--border-radius);
    cursor: pointer;
    transition: background-color var(--transition-speed);
    margin-top: 0px;
    display: block;
    vertical-align: top;
}

.subtype-button-clicked {
    background-color: #546a8b;
}
.node-dialog {
    position: absolute;
    border: 1px solid #380c8f;
    background: rgb(9, 0, 19);
    padding: 10px;
    box-shadow: 2px 2px 10px rgb(147, 128, 201);
    z-index: 10;
}

.close-dialog-btn {
    position: absolute;
    bottom: 5px;
    right: 5px;
    padding: 1px 5px;
    background-color: var(--primary-color);
    color: var(--text-color);
    border: none;
    border-radius: 2px;
    cursor: pointer;
    font-size: var(--font-size-xsmall);
}

.close-dialog-btn:hover {
    background-color: var(--close-dialog-btn-hover-color);
}

table, th, td {
    border: 1px solid var(--table-border-color);
    border-collapse: collapse;
}
th {
    background-color: var(--th-background-color); /* Example color for headers */
}


.filter-button-active {
    /* Define the color for active (pressed) state */
    background-color: var(--filter-button-active); /* Example: green background */
    color: var(--filter-button-color); /* Example: white text */
}

.close-dialog-btn:hover {
    background-color: #d32f2f;
}



.filter-button-active {
    /* Define the color for active (pressed) state */
    background-color: #4CAF50; /* Example: green background */
    color: gray; /* Example: white text */
}

.action-group-group {
    display: flex;
    flex-direction: row;
    align-items: left;
    margin-bottom: 10px;
    font-size: var(--font-size-small);
}

.action-group {
    display: flex;
    flex-direction: column;
    align-items: left;
    margin-right: 10px;
    font-size: var(--font-size-vsmall);
}
.relationships {
    display: none;
}


.actionSubmit {
    background-color: var(--actionsubmit-bg-color);
    color: var(--actionsubmit-color);
    border: none;
    border-radius: var(--border-radius);
    cursor: pointer;
    padding: 1px;
    margin: 2px;
    transition: background-color var(--transition-speed);
}

@keyframes flash {
    0%, 100% { 
        opacity: 0.681; 
        background-color: var(--actionsubmit-bg-color); /* Normal background color */
        transform: scale(1); /* Normal scale */
        border-radius: var(--border-radius);
        color:var(--property-button-color);

    }
    80% { 
        opacity: 0.961; 
        background-color: var(--close-dialog-btn-hover-color); /* Striking background color */
        transform: scale(1.01); /* Slightly larger scale */
        border-radius: var(--border-radius);
        color:var(--property-button-color);
    }
}

.flash-effect {
    animation: flash 1s linear 1; /* Change '3' to the number of flashes */
}


.header-container {
    display: flex;
    align-items: center;
}
.header-container h1 {
    margin-right: 20px;
}
.header-container a {
    margin: 0 10px;
}
.header-container small {
    margin-left: 5px;
}