<div class="accordion-content">
    <ul>
            <form action="download_file" method="post">
                <label for="euid">File EUID: </label>
        <input type="text" id="euid" name="euid"><br><br>
        
        <label for="download-type">File Creation Mode:</label> 
        <select id="download-type" name="download_type">
            <option value="dewey" selected>dewey</option>
            <option selected value="hybrid">hybrid</option>
            <option value="orig">orig</option>
        </select>
        <ul>File(s) downloaded will be named as follows:
            <ul>
            <li>dewey: <code>{euid}.{original_file_suffix}</code></li>
            <li>hybrid:<code> {euid}.{original_file_name}</code></li>
            <li>orig:<code> {original_file_name}.{original_file_name}</code></li>
            <ul> ... <b> <small>not advised: !greater risk of over-writing similar named files! use <code>*.dewey.yaml</code> file to work with original file info.</small></b>
            </ul>
        </ul></ul>
        <br>
     
        <label for="create_metadata_file">Generate dewey.yaml:</label>
        <select id="create_metadata_file" name="create_metadata_file">
            <option selected value="yes">yes</option>
            <option value="no">no</option>
        </select>
        <ul>Creates a companion <code>{downloaded_file}.dewey.yaml</code> file with key metadata assoc with the file.<ul>
             (the exhaustive view of any object can be accessed via the<a href=euid_details?euid=GT1 target=neww>EUID Details View, ex: GT1</a>)
        </ul></ul>
        <br><br>
    
        <button type="submit">Download File</button>
    </form>
    </ul>
</div>
