<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">

    {% set page_title = 'Search Error' %}
    <title>{{ page_title }}</title>

    {% set bloom_mod = 'dewey' %}

    <link rel="stylesheet" type="text/css" href="{{ style.skin_css }}">
    <link rel="stylesheet" type="text/css" href="/static/legacy/style.css">
    <script src="/static/legacy/action_buttons.js"></script>
</head>
<body>
    {% include 'legacy/bloom_header.html' %}

    <h1>Search Error</h1>
    <p>An error occurred while trying to search the files:</p>
    <p>{{ error }}</p>
    <br>
    <a href="/dewey">Go Back</a>
</body>
</html>
