@import url('https://fonts.googleapis.com/css2?family=Dosis&display=swap');


/* CSS Variables for Easy Theme Management */
:root {
    --primary-color: #e74c3c;
    --button-bg-color: #54c9cd;
    --button-border-color: #4d1ee7;
    --primary-hover-color: #7e2916;
    --actionsubmit-bg-color: rgb(204,11,139);
    --actionsubmit-color: #ffffff;
    --secondary-color: #3d6db2;
    --secondary-hover-color: #02607e;
    --accent-color: #8929c9;
    --url-color: #3d6db2;
    --addl-button-color: #3d6db2;
    --background-color: #111111;
    --container-bg-color: #0b131a;
    --panel-bg-color: #011220;
    --text-color: #ffffff;
    --font-family: 'Dosis', sans-serif; /* Replace 'Your Font Family' with your desired font */
    --font-size: 16px; /* Example size */
    --font-weight: inherit; 
    --border-radius: 4px;
    --font-size-standard: 16px;
    --font-size-medium: 20px;
    --font-size-large: 24px;
    --font-size-xlarge: 36px;
    --font-size-small: 14px;
    --font-size-xsmall: 9px;
    --transition-speed: 0.3s;
    --form-in-bg-color: lightgray;
    --form-border-color: #ccc;
    --form-color: black;
    --accordion-after-color: #777;
    --status-button-color: #d6f3f8;
    --property-button-color: #898989;
    --form-container-input-color: #ccc;
    --info-box-border-color: rgb(22, 191, 191);
    --hilight-color: rgb(2, 130, 147);
    --a: rgb(109, 241, 241);
    --a-visited: rgb(193, 33, 221);
    --a-hover: rgb(246, 11, 254); /* Change to your preferred color */
    --a-active: rgb(190, 243, 252);  
    --close-dialog-btn-hover-color: #d32f2f;
    --table-border-color: #101628;
    --th-background-color: #aeacd4;
    --filter-button-active:  #4CAF50;
    --filter-button-color: white;
    --workflow-non-step-bg-color: #02849e;
    --accordion-prefix: lightgray;
    --disabled-action-btn: #acb6b7;
}


input[type="date" i] {
    font-family: inherit;
    padding-inline-start: 1px;
    cursor: default;
    padding: inherit;
    border: inherit;
    overflow: hidden;
}

input[type="file" i] {
    font-family: inherit;
    appearance: none;
    background-color: inherit;
    cursor: default;
    align-items: baseline;
    color: inherit;
    text-overflow: ellipsis;
    text-align: start !important;
    padding: initial;
    border: inherit;
    white-space: pre;
    overflow: hidden !important;
}